@@ -10,6 +10,7 @@
 """
 import os
 import ssl
+import subprocess
 import textwrap
 
 
@@ -171,3 +172,28 @@ def test_reloader_nested_broken_imports(tmpdir, dev_server, reloader_type):
     r = requests.get(server.url)
     assert r.status_code == 200
     assert r.content == b'hello'
+
+
+def test_monkeypached_sleep(tmpdir):
+    # removing the staticmethod wrapper in the definition of
+    # ReloaderLoop._sleep works most of the time, since `sleep` is a c
+    # function, and unlike python functions which are descriptors, doesn't
+    # become a method when attached to a class. however, if the user has called
+    # `eventlet.monkey_patch` before importing `_reloader`, `time.sleep` is a
+    # python function, and subsequently calling `ReloaderLoop._sleep` fails
+    # with a TypeError. This test checks that _sleep is attached correctly.
+    script = tmpdir.mkdir('app').join('test.py')
+    script.write(textwrap.dedent('''
+    import time
+
+    def sleep(secs):
+        pass
+
+    # simulate eventlet.monkey_patch by replacing the builtin sleep
+    # with a regular function before _reloader is imported
+    time.sleep = sleep
+
+    from werkzeug._reloader import ReloaderLoop
+    ReloaderLoop()._sleep(0)
+    '''))
+    subprocess.check_call(['python', str(script)])
