@@ -67,13 +67,16 @@ r'''
     or as named parameters, pretty much like Python function calls.
 
 
-    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2014 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 '''
+from __future__ import print_function
+
 import sys
 import inspect
 import getopt
 from os.path import basename
+from werkzeug._compat import iteritems
 
 
 argument_types = {
@@ -138,7 +141,7 @@ def run(namespace=None, action_prefix='action_', args=None):
 
     try:
         optlist, posargs = getopt.gnu_getopt(args, formatstring, long_options)
-    except getopt.GetoptError, e:
+    except getopt.GetoptError as e:
         fail(str(e))
 
     specified_arguments = set()
@@ -168,7 +171,7 @@ def run(namespace=None, action_prefix='action_', args=None):
             fail('Invalid value for \'%s\': %s' % (key, value))
 
     newargs = {}
-    for k, v in arguments.iteritems():
+    for k, v in iteritems(arguments):
         newargs[k.startswith('no_') and k[3:] or k] = v
     arguments = newargs
     return func(**arguments)
@@ -176,14 +179,14 @@ def run(namespace=None, action_prefix='action_', args=None):
 
 def fail(message, code=-1):
     """Fail with an error."""
-    print >> sys.stderr, 'Error:', message
+    print('Error: %s' % message, file=sys.stderr)
     sys.exit(code)
 
 
 def find_actions(namespace, action_prefix):
     """Find all the actions in the namespace."""
     actions = {}
-    for key, value in namespace.iteritems():
+    for key, value in iteritems(namespace):
         if key.startswith(action_prefix):
             actions[key[len(action_prefix):]] = analyse_action(value)
     return actions
@@ -191,29 +194,28 @@ def find_actions(namespace, action_prefix):
 
 def print_usage(actions):
     """Print the usage information.  (Help screen)"""
-    actions = actions.items()
-    actions.sort()
-    print 'usage: %s <action> [<options>]' % basename(sys.argv[0])
-    print '       %s --help' % basename(sys.argv[0])
-    print
-    print 'actions:'
+    actions = sorted(iteritems(actions))
+    print('usage: %s <action> [<options>]' % basename(sys.argv[0]))
+    print('       %s --help' % basename(sys.argv[0]))
+    print()
+    print('actions:')
     for name, (func, doc, arguments) in actions:
-        print '  %s:' % name
+        print('  %s:' % name)
         for line in doc.splitlines():
-            print '    %s' % line
+            print('    %s' % line)
         if arguments:
-            print
+            print()
         for arg, shortcut, default, argtype in arguments:
             if isinstance(default, bool):
-                print '    %s' % (
+                print('    %s' % (
                     (shortcut and '-%s, ' % shortcut or '') + '--' + arg
-                )
+                ))
             else:
-                print '    %-30s%-10s%s' % (
+                print('    %-30s%-10s%s' % (
                     (shortcut and '-%s, ' % shortcut or '') + '--' + arg,
                     argtype, default
-                )
-        print
+                ))
+        print()
 
 
 def analyse_action(func):
@@ -262,11 +264,15 @@ def make_shell(init_func=None, banner=None, use_ipython=True):
         namespace = init_func()
         if ipython:
             try:
-                import IPython
+                try:
+                    from IPython.frontend.terminal.embed import InteractiveShellEmbed
+                    sh = InteractiveShellEmbed(banner1=banner)
+                except ImportError:
+                    from IPython.Shell import IPShellEmbed
+                    sh = IPShellEmbed(banner=banner)
             except ImportError:
                 pass
             else:
-                sh = IPython.Shell.IPShellEmbed(banner=banner)
                 sh(global_ns={}, local_ns=namespace)
                 return
         from code import interact
